#' Use Case 1 Optimal design evaluation
#' =========================================================================================
#' This example script is intended to 
#' to illustrate optimal design with PFIM and Pop ED
#'   
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Performing evaluation of study design characteristics using PFIM and PopED
#'  
#' Prepared by Mike K Smith, Pfizer Ltd, Sandwich, UK
#' 04 August 2016
#'   
#' Initialisation
#' =========================
#' Clear workspace and set working directory 
rm(list=ls(all=F))
getwd()
#' Some useful functions for working with the SEE and workspaces in the MDL-IDE
#' A system variable has been set which retrieves the directory of the workspaces
Sys.getenv("MDLIDE_WORKSPACE_HOME")
#' By typing "~" you can get to this directory
mydir <- file.path("~","UseCasesDemo","models","Design")
setwd(mydir)

#' Another function from the ddmore R package retrieves the directory that the SEE is installed in.
#' You can then navigate relative to these directories.
ddmore:::DDMORE.checkConfiguration()

#' Set name of .mdl file 
mdlfile <- "UseCase1_design1_eval.mdl"
model <- tools::file_path_sans_ext(mdlfile)

wd <- file.path(mydir,model)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,mdlfile),wd)

#' Set the working directory. 
setwd(wd)
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = wd) 
cache.path <- file.path(wd, 'cache')
dir.create(cache.path)
opts_chunk$set(cache.path=cache.path)
figure.path <- file.path(wd, 'figure')
dir.create(figure.path)
opts_chunk$set(figure.path=figure.path)

#' List files available in working directory
list.files()

#' Read the different MDL objects needed for upcoming tasks.
#' An MDL file may contain more than one object of any type 
#' (though typically only one model!) so these functions return
#' a list of all objects of that type found in the target MDL file.
#' To pick out the first, use the double square bracket notation in R
#' to retrieve the first item of the list. 
myModelObj <- getModelObjects(mdlfile)[[1]] 
myDesignObj <- getDesignObjects(mdlfile)[[1]]
myParameterObj <- getParameterObjects(mdlfile)[[1]]
myTaskPropertiesObj <- getTaskPropertiesObjects(mdlfile)[[1]]

#' Design evaluation using PFIM
#' --------------------------------
#' The Design Object settings and Task Properties settings dictate whether evaluation or 
#' optimisation of the trial design is performed.

#' The runPFIM function takes an MDL file, converts to PharmML, runs the converter and then
#' runs a created batch script to run PFIM.
pharmmlfile <- as.PharmML(mdlfile)

#' The runPFIM function takes the MDL file (or converted PharmML) as inputs, and runs the converter
#' to create the required PFIM .R files and an associated .bat file for running PFIM in R.
#' Note that PFIM is run in an independent R instance.
#' An option "run=FALSE" will perform the conversion and stop.
#' 
#+ PFIM, cache=TRUE
runPFIM(pharmmlfile=pharmmlfile, jarLocation=file.path(ddmore:::DDMORE.checkConfiguration(),"PFIM","converter"))
#+ readPFIM
myOutputFile <- myTaskPropertiesObj@EVALUATE$blocks[[1]]$TARGET_SETTINGS$output
myOutputFile <- gsub("\\\"","",myOutputFile)
cat(readLines(file.path(getwd(),"PFIM",myOutputFile) ),sep="\n")

#' Design evaluation using PopED
#' --------------------------------
#' PopED function as.poped takes a PharmML file and creates 
#' a poped.db database object ready for use with PopED.
#' We can then use PopED functions directly (natively) in R.
library(PopED)

as.poped(pharmmlfile)

#'  create plot of model without variability 
#+ Plot_model_predictions_from_PopED
plot_model_prediction(poped.db)

#' get predictions from model
popED.pred <- model_prediction(poped.db)

popED.pred[order(popED.pred$Time),]

#' evaluate initial design
FIM <- evaluate.fim(poped.db) 
FIM
det(FIM)
get_rse(FIM,poped.db)
