#' Prior example335 test 
#' =========================================================================================
#' This example script is intended to 
#' to illustrate Bayesian analysis with BUGS 
#' based on example335.mdl following prior specification in the BayesianModels_v1.1.doc
#' section 3.3.5.
#'   
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Exploratory graphical analysis
#' * Parameter estimation in Winbugs and MCMC diagnostics using coda
#'  
#' Prepared by Mike K Smith, Pfizer Ltd, Sandwich, UK
#' 04 August 2016
#' 
#' Initialisation
#' =========================
#' Clear workspace and set working directory 
#+ Initialisation
rm(list=ls(all=F))
getwd()
#' Some useful functions for working with the SEE and workspaces in the MDL-IDE
#' A system variable has been set which retrieves the directory of the workspaces
Sys.getenv("MDLIDE_WORKSPACE_HOME")
#' By typing "~" you can get to this directory
mydir <- file.path("~/UseCasesDemo/models/Priors")
setwd(mydir)

#' Another function from the ddmore R package retrieves the directory that the SEE is installed in.
#' You can then navigate relative to these directories.
ddmore:::DDMORE.checkConfiguration()

#' Set name of .mdl file 
mdlfile <- "example335.mdl"
model <- tools::file_path_sans_ext(mdlfile)
datafile <- "Testdata.csv"

wd <- file.path(mydir,model)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,datafile),wd)
file.copy(file.path(mydir,mdlfile),wd)

#' Set the working directory. 
setwd(wd)
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = wd) 
cache.path <- file.path(wd, 'cache')
dir.create(cache.path)
opts_chunk$set(cache.path=cache.path)
figure.path <- file.path(wd, 'figure')
dir.create(figure.path)
opts_chunk$set(figure.path=figure.path)

#' List files available in working directory
list.files()

#' Read the different MDL objects needed for upcoming tasks.
#' An MDL file may contain more than one object of any type 
#' (though typically only one model!) so these functions return
#' a list of all objects of that type found in the target MDL file.
#' To pick out the first, use the double square bracket notation in R
#' to retrieve the first item of the list. 
myDataObj <- getDataObjects(mdlfile)[[1]]
 
#' Exploratory Data Analysis
#' =========================
#' getDataObjects reads and parses the MDL. It doesn't actually read the data file.
#' To do so, use the ddmore function readDataObj() to create an R object (data frame) 
#' based on the MDL data object. This uses data column headers defined in the Data Object.
myData <- readDataObj(myDataObj)

#' Let's look at the first 6 lines of the data set
head(myData)
myEDAData <- myData[!is.na(myData$DV),]

library(ggplot2)
#' Plot the data using ggplot2 library 
plot1 <- ggplot(aes(y=DV,x=TIME, group=ID), data=myEDAData) +
				geom_line()+
				geom_point() + 
				labs(y="Concentration" ,x="Time (hours)")
#+ ExploratoryAnalysis
print(plot1)

#' Model Development
#' =========================
#' ESTIMATE model parameters in WINBUGS
#' -------------------------
#' Assembling the new MOG for Winbugs. Note that we reuse the data and model. 
#' Two main changes are made: selecting the appropriate Task Properties Object for BUGS
#' and exchanging the Prior Object instead of the Parameter Object.

#+ BUGSestimate, cache=TRUE
# BUGS <- estimate(mdlfile, target="winbugs", subfolder="WinBUGS", preprocessSteps=list(prepareWinbugsDs))

#' Alternatively the runWinBUGS function is provided with the ddmore R package to allow the user
#' some additional control over BUGS execution.

#+ BUGS, cache=TRUE
 BUGS<-runWinBUGS(mdlfile,subfolder="WinBUGS")

# BUGS <- LoadSOObject(file.path("WinBUGS","InstallationTest_DEQ_BUGS.SO.xml"))

getPopulationParameters(BUGS)$Bayesian
getPopulationParameters(BUGS, what="estimates")$Bayesian # o other options,  what= precisions, intervals, all
# it is possible also to omit $Bayesian

#' Here we use the coda package to examine and assess MCMC convergence.
library(coda)

#' By default the parameters monitored by BUGS are all parameters specified with
#' random variable distributions in the PRIOR_VARIABLE_DEFINITION. In this example
#' these are specified on the log scale. The following code transforms them back to
#' the natural scale. Note that in the public release, MCMC nodes for monitoring 
#' will be specified via the Task Properties Object.

parameters_BUGS<- getPopulationParameters(BUGS, what="estimates")$Bayesian
#' How to retrieve parameter names from the Task Properties Object
#estPars <- getTaskPropertiesObjects(mdlfile)[[1]]@ESTIMATE$blocks[[1]]$TARGET_SETTINGS$parameters
#estPars <- gsub("\\\"","", estPars)
#estPars <- gsub(" ", "", estPars)
#estPars <- unlist(strsplit(estPars, split=","))

#' Since there are very few parameters and they are consistent across the Priors example models:
estPars <- c("POP_K","POP_V","POP_T","OMEGA_K","OMEGA_V","OMEGA_T")

#' coda MCMC trace and density plots
#+ MCMC_Diagnostics
#to read only one chain
BUGSoutputPath <- file.path(getwd(),"WinBUGS")
coda_out <- read.coda(output.file=file.path(BUGSoutputPath,"output1.txt"), index.file=file.path(BUGSoutputPath,"outputIndex.txt"), quiet=T)
coda_pars <- coda_out[,estPars]
#to read more chains (3 in this case) use read.coda.interactive()
#coda_out <- read.coda.interactive()
# Enter in the order:
  #outputIndex.txt
  #output1.txt  
  #output2.txt  
  #output2.txt
## summary statistics
summary(coda_pars)
## trace and density plots
plot(coda_pars, ask=F)
## gelman-rubin diagnostics (if at least 2 chains)
#gelman.diag(coda_out)
#gelman.plot(coda_out)
