#' Guided-demo: Exploring 'ddmore' R package functionality of time to event models
#' =========================================================================================
#' UseCase14_1 : ODE + time to event model for exact and right censored information
#' ---------------------------------
#' This example script is intended to illustrate how to use the 'ddmore' R package
#' to perform a M&S workflow using the DDMoRe Standalone Execution Environment (SEE). 
#'   
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Parameter estimation with Monolix
#' * Using MLE values from Monolix to simulate new observed values using the simulx function
#'   in the mlxR package.
#' 
#' To run a task, select with the cursor any code lines you wish to execute and press
#' CTRL+R+R in your keyboard.
#' An HTML file containing the commands in this file and associated output will be provided
#' to allow the user to compare the results


#' Initialisation
#' =========================
#+ Initialisation
#' Clear workspace and set working directory under 'UsesCasesDemo' project
rm(list=ls(all=FALSE))
mydir <- file.path(Sys.getenv("MDLIDE_WORKSPACE_HOME"),"UseCasesDemo")
setwd(mydir)

#' Create a working directory under 'models' folder where results are stored
uc<-"UseCase14_1"
datafile <- "warfarin_TTE_exact.csv"
mdlfile <- paste0(uc,".mdl")

#' Create a new folder to be used as working directory and where results will be stored
wd <- file.path(mydir,uc)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,"models", datafile),wd)
file.copy(file.path(mydir,"models",mdlfile),wd)

#' Set the working directory. 
setwd(file.path(mydir,uc))
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = file.path(mydir,uc)) 

#' List files available in working directory
list.files()


#' Model Development
#' =========================

#' ESTIMATE model parameters using Monolix
#' ------------------------- 
#' The ddmore "estimate" function translates the contents of the .mdl file to 
#' a target language and then estimates parameters using the target software. After estimation, 
#' the output is converted to a Standardised Output object which is saved in a .SO.xml file.
#'   
#' Translated files and Monolix output will be returned in the ./Monolix subfolder.
#' The Standardised Output object (.SO.xml) is read and parsed into an R object called "mlx" 
#' of (S4) class "StandardOutputObject".
#+ Monolix Estimation
mlx <- estimate(mdlfile, target="MONOLIX", subfolder="Monolix")
slotNames(mlx)

#' The ddmore "LoadSOObj" reads and parsed existing Standardise Output Objects
#' 
#'  mlx <- LoadSOObject("Monolix/UseCase14_1.SO.xml")
#' 
#' The ddmore function "getPopulationParameters" extracts the Population Parameter values from 
#' an R object of (S4) class "StandardOutputObject" and returns the estimates. 
#' See documentation for getPopulationParameters to see other arguments and settings for this function.  
parameters_mlx <- getPopulationParameters(mlx, what="estimates")$MLE
print(parameters_mlx)
print(getPopulationParameters(mlx, what="precisions"))

print(getEstimationInfo(mlx))



#' Estimation with NONMEM
#' -------------------------
#' NONMEM estimation is not possible for this use case. In the current version of the framework, 
#' there is no place to define initial conditions for an ODE which are not structural parameters so that the model runs interoperably.
#' See UC14_2 for an alternative implementation that can be used with NONMEM. 

#' Simulation using simulx
#' --------------------------------
#' The mlxR package has been developed to visualize and explore models that are encoded in
#' MLXTRAN or PharmML. 
#' The ddmore function as.PharmML translates an MDL file (extension .mdl) to its PharmML
#' representation. The output file (extension .xml) is saved in the working directory.
myPharmML <- as.PharmML(mdlfile)

#' Use parameter values from the Monolix estimation
parValues <- getPopulationParameters(mlx, what="estimates")$MLE
parValues <- parValues[c("POP_BTATRT","POP_HBASE")]

#' Add the treatment covariate
p1 <- c(parValues, TRT = 1)
p2 <- c(parValues, TRT = 4)

#' Parameter values used in simulation
print(p1)
print(p2)

#' Simulate events
h <- list(name='HAZ', time=seq(0, 60, by=1))
e <- list(name='Y', time=0)

#' Simulate 100 subjects within each treatment group
g1 <- list( size = 100, level = 'longitudinal',  parameter = p1)
g2 <- list( size = 100, level = 'longitudinal',  parameter = p2)

#' Call simulx
res <- simulx(model = myPharmML,
			   group = list(g1,g2),
			   output = list(h,e))

#' Kaplan-Meyer plot of the simulated time-to-event data
pl1 <- kmplotmlx(res$Y, level=0.9)
print(pl1)
