#' UseCase17_1 : Steady-state model
#' ---------------------------
#' This example script is intended to illustrate how to use the 'ddmore' R package
#' to perform a M&S workflow using the DDMoRe Standalone Execution Environment (SEE). 
#'  
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Parameter estimation with NONMEM
#' * Evaluation of the model executed in NONMEM using Xpose
#' * Change estimation method to FOCEI and re-estimation of the model via PsN
#' * Evaluation of the model executed in NONMEM using Xpose
#' * Updating parameter estimates in the MDL Parameter Object 
#'   using MLE values from NONMEM
#' * Performing a Visual Predictive Check (VPC) in PsN
#' * Using MLE values from NONMEM to simulate new observed values using the simulx function
#'   in the mlxR package.
#' 
#' To run a task, select with the cursor any code lines you wish to execute and press
#' CTRL+R+R in your keyboard.
#' An HTML file containing the commands in this file and associated output will be provided
#' to allow the user to compare the results


#' Initialisation
#' =========================
#+ Initialisation
#' Clear workspace and set working directory under 'UsesCasesDemo' project
rm(list=ls(all=F))
mydir <- file.path(Sys.getenv("MDLIDE_WORKSPACE_HOME"),"UseCasesDemo")
setwd(mydir)

#' Create a working directory under 'models' folder where results are stored
uc<-"UseCase17_1"
datafile <- "warfarin_conc_SSADDL.csv"
mdlfile <- paste0(uc,".mdl")

#' Create a new folder to be used as working directory and where results will be stored
wd <- file.path(mydir,uc)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,"models", datafile),wd)
file.copy(file.path(mydir,"models",mdlfile),wd)

#' Set the working directory. 
setwd(file.path(mydir,uc))
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = file.path(mydir,uc)) 

#' List files available in working directory
list.files()




#' Model Development
#' =========================

#' ESTIMATE model parameters using Monolix
#' -------------------------
#' The version of Monolix used in the SEE (4.3.2) does not support a combination of 
#' steady-state dosing (SS) and additional dosing records (ADDL). Therefore, this 
#' Use Case will not be converted to MLXTRAN.

#' SAEM Estimation with NONMEM
#' -------------------------
#' By default, a covariance step is not run when estimating in NONMEM. 
#' To see how it can be requested, see UseCase1_1.mdl
#+ NONMEM Estimation SAEM
NM <- estimate(mdlfile, target="NONMEM", subfolder="NONMEM")

#' Load previous results
#' 
#'  NM <- LoadSOObject("NONMEM/UseCase17_1.SO.xml")
#' 
#' Results from NONMEM
parameters_nm <- getPopulationParameters(NM, what="estimates")
print(parameters_nm)

print(getEstimationInfo(NM))


#' Xpose diagnostics using NONMEM output
#' -------------------------
#+ GOF for NONMEM Estimation
nm.xpdb<-as.xpdb(NM,datafile)

#' Perform some basic goodness of fit (graphs are exported to PDF file)
print(basic.gof(nm.xpdb))
print(ind.plots(nm.xpdb))
print(parm.hist(nm.xpdb))

#' Export graphs to a PDF file
pdf("GOF_NM.pdf")
print(basic.gof(nm.xpdb))
print(ind.plots(nm.xpdb))
print(parm.hist(nm.xpdb))
dev.off()

#' Change estimation method to FOCEI (for speed)
#' ------------------------------------------------------------
#+ Change_estimation_method
#' MDL Objects can be manipulated from R to change for example the estimation algorithm
myTaskProperties <- getTaskPropertiesObjects(mdlfile)[[1]]
myNewTaskProperties <- myTaskProperties
myNewTaskProperties@ESTIMATE$algo <- "focei"

#' Assembling the new MOG. Note that we reuse the data and model from the previous run.  
myNewerMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = getParameterObjects(mdlfile)[[1]], 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = myNewTaskProperties)

#' We can then write the MOG back out to an .mdl file.
mdlfile.FOCEI <- paste0(uc,"_FOCEI.mdl")
writeMogObj(myNewerMOG,mdlfile.FOCEI)

#' Test estimation using this new MOG.
#+ PsN (NONMEM) Estimation FOCEI
NM.FOCEI <- estimate(mdlfile.FOCEI, target="PsN", subfolder="NONMEM_FOCEI")

#' Load previous results 
#NM.FOCEI <- LoadSOObject("NONMEM_FOCEI/UseCase17_1_FOCEI.SO.xml")
#' 
#' Results from NONMEM should be comparable to previous results
print(getPopulationParameters(NM.FOCEI,  what="estimates"))
print(parameters_nm)


#' Xpose diagnostics using NONMEM output
#' -------------------------
#+ GOF PsN estimation
nmfocei.xpdb<-as.xpdb(NM.FOCEI,datafile)

#' Basic diagnostics for NONMEM fit.
print(basic.gof(nmfocei.xpdb))

#' Export graphs to a PDF file
pdf("GOF_NM_FOCEI.pdf")
print(basic.gof(nmfocei.xpdb))
dev.off()

#' VPC of model
#' --------------------------------
#' Before running the VPC with PsN we must update the (initial) values in the MDL Parameter Object
#' MLE estimates from previous step can be used 
#+ Update parObj
structuralPar <- getPopulationParameters(NM.FOCEI, what="estimates",block='structural')$MLE
variabilityPar <- getPopulationParameters(NM.FOCEI, what="estimates",block='variability')$MLE

#' Update the parameter object using the ddmore "updateParObj" function.
#' This function updates an R object of (S4) class "parObj".
#' The user chooses which block to update, what items within that block,
#' and what to replace those items with. 
#' NOTE: that updateParObj can only update attributes which ALREADY EXIST in the MDL Parameter Object for that item.
#' This ensures that valid MDL is preserved.
myParObj <- getParameterObjects(mdlfile)[[1]]
myParObjUpdated <- updateParObj(myParObj,block="STRUCTURAL",
		item=names(structuralPar),
		with=list(value=structuralPar))
myParObjUpdated <- updateParObj(myParObjUpdated,block="VARIABILITY",
		item=names(variabilityPar),
		with=list(value=variabilityPar))

#' Check that the appropriate initial values have been updated to the MLE values from the previous fit.
print(myParObjUpdated@STRUCTURAL)
print(myParObjUpdated@VARIABILITY)

#' Assembling the new MOG. Note that we reuse the data and model from the previous run.  
myVPCMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = myParObjUpdated, 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = getTaskPropertiesObjects(mdlfile)[[1]])

#' We can then write the MOG back out to an .mdl file.
mdlfile.VPC <- paste0(uc,"_VPC.mdl")
writeMogObj(myVPCMOG,mdlfile.VPC)

#' Similarly as above, ddmore "VPC.PsN" function can be used to run a VPC using PsN as target tool
#+ VPC, cache=TRUE
vpcFiles <- VPC.PsN(mdlfile.VPC,samples=20, seed=12345,
		vpcOptions ="-n_simulation=10 -auto_bin=10",
		subfolder="VPC", plot=TRUE) 

#' To replay the visualisation using information from the VPC SO file
pdf(paste0(uc,"_VPC.pdf"))
print(xpose.VPC(vpc.info= file.path("./VPC",vpcFiles@RawResults@DataFiles$PsN_VPC_results$path),
				vpctab= file.path("./VPC",vpcFiles@RawResults@DataFiles$PsN_VPC_vpctab$path),
				main="VPC warfarin"))
dev.off()

#' Simulation using simulx
#' --------------------------------
#' The mlxR package has been developed to visualize and explore models that are encoded in
#' MLXTRAN or PharmML 
#' The ddmore function as.PharmML translates an MDL file (extension .mdl) to its PharmML
#' representation. The output file (extension .xml) is saved in the working directory.
#+ Simulation via simulx
myPharmML <- as.PharmML(mdlfile)

#' Use parameter values from the FOCEI estimation
parValues <- getPopulationParameters(NM.FOCEI, what="estimates")$MLE

#' Simulate for the typical weight of 70. 
#' Recall that logtWT = log(WT/70). 
p <- c(parValues,logtWT=0)

#' Parameter values used in simulation
print(p)

#' Simulate PK parameters for individuals
ind <- list(name = c('KA','CL','V'))

#' Simulate predicted (CC) and observed concentration values (Y)
f   <- list( name = c('CC'), time = seq(0,to=50,by=1))
y   <- list( name = c('Y'), time = c(0, 0.5, 1, 2, 3, 4, 6, 8, 12, 24, 36, 48))

#' Simulate for a dose of 100mg given at time 0 (oral administration)
adm <- list(type = 1, time = 0, amount = 100)

#' Simulate 12 subjects
g <- list( size = 12, level = 'individual', treatment=adm)

#' Call simulx
res  <- simulx(model =myPharmML,
		       parameter = p,
			   group = g,
			   output = list(ind,f,y))

#' Simulated parameter values for each individual
print(res$parameter)

#' Plot simulated results
plot(ggplot() + 
				geom_line(data=res$CC, aes(x=time, y=CC, colour=id)) +
				geom_point(data=res$Y, aes(x=time, y=Y, colour=id)) +
				xlab("time (h)") + ylab("concentration") )

#' Simulate 1000 subjects - with simulx this is a QUICK process!
#+ VPC via simulx
g <- list( size = 1000, level = 'individual',  treatment = adm)

#' Call simulx
res.1000  <- simulx(model =myPharmML,
		            parameter = p,
					group = g,
					output = list(ind,y))

#' Plot of observed concentrations (with residual error)
#'  band defines the percentile bands displayed
#'    level = range of values to examine (in %)
#'            100 = full range of values
#'    number = number of bins within the level range.
print(prctilemlx(res.1000$Y,band=list(number=9, level=90)))

#' Table of the same information
print(prctilemlx(res.1000$Y,band=list(number=10, level=100), plot=FALSE)$y)
