#' UseCase3_1 : PKPD model for warfarin population pharmacokinetics and PCA
#' ========================================================================
#'  Variant on UseCase3 : PK parameters passed as model inputs via "use is variable"
#' -----------------------------------------------------------------------
#' This example script is intended to illustrate how to use the 'ddmore' R package
#' to perform a M&S workflow using the DDMoRe Standalone Execution Environment (SEE). 
#'    
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Read and parse MDL files in R.
#' * Exploratory graphical analysis
#' * Parameter estimation with Monolix
#' * Evaluation of the model executed in Monolix 
#' * Change estimation method to FOCEI and re-estimation of the model via PsN
#' * Updating parameter estimates in the MDL Parameter Object 
#'   using MLE values from NONMEM
#' * Performing a Visual Predictive Check (VPC) in PsN
#' * Using MLE values from Monolix to simulate new observed values using the simulx function
#'   in the mlxR package.
#' 
#' To run a task, select with the cursor any code lines you wish to execute and press
#' CTRL+R+R in your keyboard.
#' An HTML file containing the commands in this file and associated output will be provided
#' to allow the user to compare the results 


#' Initialisation
#' =========================
#' Clear workspace and set working directory under 'models' folder
#+ Initialisation
#' Clear workspace and set working directory under 'UsesCasesDemo' project
rm(list=ls(all=F))
mydir <- file.path(Sys.getenv("MDLIDE_WORKSPACE_HOME"),"UseCasesDemo")
setwd(mydir)

#' Set name of .mdl file and dataset for future tasks
uc <- "UseCase3_1"
datafile <- "warfarin_conc_pca_PKparam.csv"
mdlfile <- paste0(uc,".mdl")

#' Create a new folder to be used as working directory and where results will be stored
wd <- file.path(mydir,uc)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,"models", datafile),wd)
file.copy(file.path(mydir,"models",mdlfile),wd)

#' Set the working directory. 
setwd(file.path(mydir,uc))
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = file.path(mydir,uc)) 
cache.path <- file.path(mydir,uc, 'cache')
dir.create(cache.path)
opts_chunk$set(cache.path=cache.path)
figure.path <- file.path(mydir,uc, 'figure')
dir.create(figure.path)
opts_chunk$set(figure.path=figure.path)

#' List files available in working directory
list.files()


#' Introduction to 'ddmore' R package
#' ==================================
#' 
#' View objects within the .mdl file
#' ---------------------------
#' Use 'ddmore' function getModelObjects() to retrieve model object(s) 
#' from an existing .mdl file. This function reads the MDL in an .mdl file
#' and parses the MDL code for each MDL Object into an R list of objects of
#' appropriate types with names corresponding to the MDL Object names given in the file.
#+ Read MDL file using the ddmore R package
myMDLObj <- getMDLObjects(mdlfile)
length(myMDLObj)
names(myMDLObj)

#' Use 'ddmore'  function getDataObjects() to retrieve only data object(s) from an existing .mdl file.
#' This function returns a list of Parameter Object(s) from which we select the first element.
#' Hover over the variable name to see its structure
myDataObj <- getDataObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getParameterObjects() to retrieve only parameter object(s) from an existing .mdl file
myParObj <- getParameterObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getModelObjects() to retrieve only model object(s) from an existing .mdl file.
myModObj <- getModelObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getTaskPropertiesObjects() to retrieve only task properties object(s) from an existing .mdl file
myTaskObj <- getTaskPropertiesObjects(mdlfile)[[1]]



#' Exploratory Data Analysis
#' =========================
#' Recall that getDataObjects only reads the MDL code from the .mdl file.
#' Use 'ddmore' function readDataObj() to create an R object from the MDL data object.
#+ Exploratory Data Analysis
myData <- readDataObj(myDataObj)

#' Let's look at the first 6 lines of the data set
head(myData)

#' Extract only observation records
myEDAData<-myData[myData$MDV==0,]

#' Open an R window to record and access all your plots
windows(record=TRUE) 

#' Now plot the data using xyplot from the lattice library 
plot3 <- xyplot(DV~TIME,groups=ID,data=myEDAData,subset=DVID==2,type="b",ylab="PCA",xlab="Time (h)")
print(plot3)
plot4 <- xyplot(DV~TIME|ID,data=myEDAData,subset=DVID==2,type="b",layout=c(3,4),ylab="PCA",xlab="Time (h)",scales=list(relation="free"))
print(plot4)

#' Export the results in a PDF file
pdf(paste0(uc,"_EGA.pdf"))
 print(plot3)
 print(plot4)
dev.off()

#' Model Development
#' =========================

#' ESTIMATE model parameters using Monolix
#' -------------------------
#' The ddmore "estimate" function translates the contents of the .mdl file to 
#' a target language and then estimates parameters using the target software. After estimation, 
#' the output is converted to a Standardised Output object which is saved in a .SO.xml file.
#'   
#' Translated files and Monolix output will be returned in the ./Monolix subfolder.
#' The Standardised Output object (.SO.xml) is read and parsed into an R object called "mlx" 
#' of (S4) class "StandardOutputObject".
#+ Monolix Estimation, cache=TRUE
mlx <- estimate(mdlfile, target="MONOLIX", subfolder="Monolix")
slotNames(mlx)
#' The ddmore "LoadSOObj" reads and parsed existing Standardise Output Objects
#' 
#'  mlx <- LoadSOObject("Monolix/UseCase3_1.SO.xml")
#' 
#' The ddmore function "getPopulationParameters" extracts the Population Parameter values from 
#' an R object of (S4) class "StandardOutputObject" and returns the estimates. 
#' See documentation for getPopulationParameters to see other arguments and settings for this function. 
parameters_mlx <- getPopulationParameters(mlx, what="estimates")$MLE
print(parameters_mlx)
print(getPopulationParameters(mlx, what="precisions"))

print(getEstimationInfo(mlx))

#' Perform model diagnostics for the base model using Xpose functions (graphs are exported to PDF)
#' -------------------------

#' There is currently a bug with as.xpdb and Monolix in UseCase3_1. 
#' Therefore, goodness-of-fit plots are created manually from the standardised output.
#' Later in the script, Xpose functionality on UseCase3 is shown for NONMEM. 
#' 
#' Use 'ddmore' function as.xpdb() to create an Xpose database object from
#' the standardised output object, regardless of target software used for estimation.
#' Users can then call xpose functions directly.  
 mlx.xpdb<-as.xpdb(mlx,datafile)
#' We can then call Xpose functions referencing this mlx.xpdb object as the input. 
#' Perform some basic goodness of fit (graphs are exported to PDF file)
pdf("GOF_MLX.pdf")
basic.gof(mlx.xpdb,by="DVID",subset="DVID==2")
ind.plots(mlx.xpdb,subset="DVID==2")
dev.off()

##+ GOFforMonolixEstimation
myPredictions <- mlx@Estimation@Predictions@data
myPredictions <- apply(myPredictions,2,as.numeric)

myXPDB <- merge(myEDAData, myPredictions) 
plot(ggplot() + ggtitle("PCA level") +
				geom_abline() +
				geom_point(data=myXPDB[myXPDB$DVID==2,], aes(x=PRED, y=DV)) +
				xlab("Population predictions") + ylab("Observations") )
plot(ggplot() + ggtitle("PCA level") +
				geom_abline() +
				geom_point(data=myXPDB[myXPDB$DVID==2,], aes(x=IPRED, y=DV)) +
				xlab("Individual predictions") + ylab("Observations") )

#' Change estimation method to FOCEI (for speed)
#' ------------------------------------------------------------
#' MDL Objects can be manipulated from R to change for example the estimation algorithm
#+ Change Estimation Method
myTaskProperties <- getTaskPropertiesObjects(mdlfile)[[1]]
myNewTaskProperties <- myTaskProperties
myNewTaskProperties@ESTIMATE$algo <- "focei"

#' Assembling the new MOG. Note that we reuse the data and model from the previous run.  
myNewerMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = getParameterObjects(mdlfile)[[1]], 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = myNewTaskProperties)

#' We can then write the MOG back out to an .mdl file.
mdlfile.FOCEI <- paste0(uc,"_FOCEI.mdl")
writeMogObj(myNewerMOG,mdlfile.FOCEI)

#' Test estimation using this new MOG.
#+ PsN (NONMEM) Estimation FOCEI, cache=TRUE
NM.FOCEI <- estimate(mdlfile.FOCEI, target="PsN", subfolder="NONMEM_FOCEI")
#' Load previous results
#' 
#' NM.FOCEI <- LoadSOObject("NONMEM_FOCEI/UseCase3_1_FOCEI.SO.xml")
#' 
#' Results from NONMEM should be comparable to previous results
print(getPopulationParameters(NM.FOCEI,  what="estimates"))



#' VPC of model
#' --------------------------------
#' Before running the VPC with PsN we must update the (initial) values in the MDL Parameter Object
#' MLE estimates from previous step can be used 
#+ Updata parObj
structuralPar <- getPopulationParameters(NM.FOCEI, what="estimates",block='structural')$MLE
variabilityPar <- getPopulationParameters(NM.FOCEI, what="estimates",block='variability')$MLE


#' Update the parameter object using the ddmore "updateParObj" function.
#' This function updates an R object of (S4) class "parObj".
#' The user chooses which block to update, what items within that block,
#' and what to replace those items with. 
#' NOTE: that updateParObj can only update attributes which ALREADY EXIST in the MDL Parameter Object for that item.
#' This ensures that valid MDL is preserved.
myParObj <- getParameterObjects(mdlfile)[[1]]
myParObjUpdated <- updateParObj(myParObj,block="STRUCTURAL",
		item=names(structuralPar),
		with=list(value=structuralPar))
myParObjUpdated <- updateParObj(myParObjUpdated,block="VARIABILITY",
		item=names(variabilityPar),
		with=list(value=variabilityPar))

#' Check that the appropriate initial values have been updated to the MLE values from the previous fit.
print(myParObjUpdated@STRUCTURAL)
print(myParObjUpdated@VARIABILITY)

#' Assembling the new MOG. Note that we reuse the data, model and tasks from the previous run.   
myVPCMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = myParObjUpdated, 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = getTaskPropertiesObjects(mdlfile)[[1]])

#' We can then write the MOG back out to an .mdl file.
mdlfile.VPC <- paste0(uc,"_VPC.mdl")
writeMogObj(myVPCMOG,mdlfile.VPC)

#' Similarly as above, ddmore "VPC.PsN" function can be used to run a VPC using PsN as target tool
#+ VPC, cache=TRUE
vpcFiles <- VPC.PsN(mdlfile.VPC,samples=20, seed=12345,
		vpcOptions ="-n_simulation=10 -auto_bin=5,8",
		subfolder="VPC", plot=FALSE) 

#' To replay the visualisation using information from the VPC SO file
pdf(paste0(uc,"_VPC.pdf"))
 print(xpose.VPC(vpc.info= file.path("./VPC",vpcFiles@RawResults@DataFiles$PsN_VPC_results$path),
		vpctab= file.path("./VPC",vpcFiles@RawResults@DataFiles$PsN_VPC_vpctab$path),
		main="VPC warfarin PK/PD"))
dev.off()

#' Simulation using simulx
#' --------------------------------
#' The mlxR package has been developed to visualize and explore models that are encoded in
#' MLXTRAN or PharmML. 
#' The ddmore function as.PharmML translates an MDL file (extension .mdl) to its PharmML
#' representation. The output file (extension .xml) is saved in the working directory.
#+ Simulation via simulx
myPharmML <- as.PharmML(mdlfile)

#' Use parameter values from the Monolix estimation
parValues <- getPopulationParameters(mlx, what="estimates")$MLE

#' Simulate for the typical weight of 70. 
#' Recall that logtWT = log(WT/70). 
p <- c(parValues,WT=70,CL=0.139,V=8,KA=2.64,TLAG=0.976)

#' Parameter values used in simulation
print(p)

#' Simulate for a dose of 100mg given at time 0 into the GUT (oral administration)
adm <- list( time = 0, amount = 100)

#' Simulate PK parameters for individuals
ind <- list(name = c('TEQ','C50','PCA0','EMAX'))

#' Simulate predicted (CC) and observed concentration values (CP_obs), predicted (PCA) and observed PCA (PCA_obs)
f2   <- list( name = c('PCA'), time = seq(0,to=50,by=1))
y2   <- list( name = c('PCA_obs'), time = c(0, 0.5, 1, 2, 3, 4, 6, 8, 12, 24, 36, 48))

#' Simulate 12 subjects
g <- list( size = 12, level = 'individual',  treatment = adm)

#' Call simulx
res  <- simulx(model = myPharmML,
		       parameter = p,
			   group = g,
			   output = list(ind,f2,y2))

#' Simulated parameter values for each individual
print(res$parameter)

#' Plot simulated results
plot(ggplot() + 
				geom_line(data=res$PCA, aes(x=time, y=PCA, colour=id)) +
				geom_point(data=res$PCA_obs, aes(x=time, y=PCA_obs, colour=id)) +
				xlab("time (h)") + ylab("PCA") )


#' Simulate 1000 subjects - with simulx this is a QUICK process!
#+ VPC via simulx
g <- list( size = 1000, level = 'individual',  treatment = adm)

#' Call simulx
res.1000  <- simulx(model =myPharmML,
		            parameter = p,
					group = g,
					output = list(ind,f2,y2))

#' Plot of predicted concentrations
#'  band defines the percentile bands displayed
#'    level = range of values to examine (in %)
#'            100 = full range of values
#'    number = number of bins within the level range.

#' Plot of observed PCA levels (with residual error)
prctilemlx(res.1000$PCA_obs,band=list(number=9, level=90))

#' Table of the same information
prctilemlx(res.1000$PCA_obs,band=list(number=10, level=100), plot=F)$y

