#' UseCase4_3 : Warfarin population pharmacokinetic model for  oral and infusion administration routes 
#' ==================================================================================================
#' Variant on UseCase4 : Using direct COMPARTMENTS to define input processes alongside ODEs
#' -----------------------------------------------------------------------------------
#' This example script is intended to illustrate how to use the 'ddmore' R package
#' to perform a M&S workflow using the DDMoRe Standalone Execution Environment (SEE). 
#'    
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Parameter estimation in MONOLIX
#' * Evaluation of the model executed in Monolix using Xpose
#' * Change estimation method to FOCEI and re-estimation of the model via PsN
#' * Evaluation of the model executed in NONMEM using Xpose
#' * Updating parameter estimates in the MDL Parameter Object 
#'   using MLE values from NONMEM
#' * Performing a Visual Predictive Check (VPC) in PsN
#' * Using MLE values from NONMEM to simulate new observed values using the simulx function
#'   in the mlxR package.
#' 
#' To run a task, select wiht the coursor any code lines you wish to execute and press
#' CTRL+R+R in your keyboard.
#' An HTML file containing the commands in this file and associated output will be provided
#' to allow the user to compare the results


#' Initialisation
#' =========================
#+ Initialisation
#' Clear workspace and set working directory under 'UsesCasesDemo' project
rm(list=ls(all=F))
mydir <- file.path(Sys.getenv("MDLIDE_WORKSPACE_HOME"),"UseCasesDemo")
setwd(mydir)

#' Create a working directory under 'models' folder where results are stored
uc<-"UseCase4_3"
datafile <- "warfarin_infusion_oral.csv"
mdlfile <- paste0(uc,".mdl")

#' Create a new folder to be used as working directory and where results will be stored
wd <- file.path(mydir,uc)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,"models", datafile),wd)
file.copy(file.path(mydir,"models",mdlfile),wd)

#' Set the working directory. 
setwd(file.path(mydir,uc))
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = file.path(mydir,uc)) 

#' List files available in working directory
list.files()




#' Model Development
#' =========================

#' ESTIMATE model parameters using Monolix
#' -------------------------
#+ Monolix Estimation
mlx <- estimate(mdlfile, target="MONOLIX", subfolder="Monolix")
slotNames(mlx)
#' The ddmore "LoadSOObj" reads and parsed existing Standardise Output Objects
#' 
# mlx <- LoadSOObject("Monolix/UseCase4_3.SO.xml")
#' 
#' The ddmore function "getPopulationParameters" extracts the Population Parameter values from 
#' an R object of (S4) class "StandardOutputObject" and returns the estimates. 
#' See documentation for getPopulationParameters to see other arguments and settings for this function.  
parameters_mlx <- getPopulationParameters(mlx, what="estimates")
print(parameters_mlx)
print(getPopulationParameters(mlx, what="precisions"))

print(getEstimationInfo(mlx))


#' Perform model diagnostics for the base model using Xpose functions (graphs are exported to PDF)
#' -------------------------
#' Use 'ddmore' function as.xpdb() to create an Xpose database object from
#' the Standardised Output object, regardless of target software used for estimation.
#+ GOF for Monolix Estimation
mlx.xpdb<-as.xpdb(mlx,datafile)

#' We can then call Xpose functions referencing this mlx.xpdb object as the input. 
#' Perform some basic goodness of fit
print(basic.gof(mlx.xpdb))
print(ind.plots(mlx.xpdb))

#' Export results to PDF file
pdf("GOF_MLX.pdf")
print(basic.gof(mlx.xpdb))
print(ind.plots(mlx.xpdb))
dev.off()

#' Change estimation method to FOCEI (for speed) in NONMEM
#' ------------------------------------------------------------
#+ Change_estimation_method
#' MDL Objects can be manipulated from R to change for example the estimation algorithm
myTaskProperties <- getTaskPropertiesObjects(mdlfile)[[1]]
myNewTaskProperties <- myTaskProperties
myNewTaskProperties@ESTIMATE$algo <- "focei"

#' Assembling the new MOG. Note that we reuse the data and model from the previous run.  
myNewerMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = getParameterObjects(mdlfile)[[1]], 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = myNewTaskProperties)

#' We can then write the MOG back out to an .mdl file.
mdlfile.FOCEI <- paste0(uc,"_FOCEI.mdl")
writeMogObj(myNewerMOG,mdlfile.FOCEI)

#' Test estimation using this new MOG in NONMEM.
#+ PsN (NONMEM) Estimation FOCEI
NM.FOCEI <- estimate(mdlfile.FOCEI, target="PsN", subfolder="NONMEM_FOCEI")
#' Load previous results
#' 
# NM.FOCEI <- LoadSOObject("NONMEM_FOCEI/UseCase4_3_FOCEI.SO.xml")
#' 
#' Results from NONMEM should be comparable to previous results
print(getPopulationParameters(NM.FOCEI,  what="estimates"))

#' Xpose diagnostics using NONMEM output
#' --------------------------------------
#+ GOF PsN estimation
nmfocei.xpdb<-as.xpdb(NM.FOCEI,datafile)

#' Basic diagnostics for NONMEM fit.
print(basic.gof(nmfocei.xpdb))

#' Export graphs to a PDF file
pdf("GOF_NM_FOCEI.pdf")
print(basic.gof(nmfocei.xpdb))
dev.off()


#' VPC of model
#' --------------------------------
#' Before running the VPC with PsN we must update the (initial) values in the MDL Parameter Object
#' MLE estimates from previous step can be used 
#+ Update parObj
structuralPar <- getPopulationParameters(NM.FOCEI, what="estimates",block='structural')$MLE
variabilityPar <- getPopulationParameters(NM.FOCEI, what="estimates",block='variability')$MLE

#' Update the parameter object using the ddmore "updateParObj" function.
#' This function updates an R object of (S4) class "parObj".
#' The user chooses which block to update, what items within that block,
#' and what to replace those items with. 
#' NOTE: that updateParObj can only update attributes which ALREADY EXIST in the MDL Parameter Object for that item.
#' This ensures that valid MDL is preserved.
myParObj <- getParameterObjects(mdlfile)[[1]]
myParObjUpdated <- updateParObj(myParObj,block="STRUCTURAL",
		item=names(structuralPar),
		with=list(value=structuralPar))
myParObjUpdated <- updateParObj(myParObjUpdated,block="VARIABILITY",
		item=names(variabilityPar),
		with=list(value=variabilityPar))

#' Check that the appropriate initial values have been updated to the MLE values from the previous fit.
print(myParObjUpdated@STRUCTURAL)
print(myParObjUpdated@VARIABILITY)

#' Assembling the new MOG. Note that we reuse the data and model from the previous run.  
myVPCMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = myParObjUpdated, 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = getTaskPropertiesObjects(mdlfile)[[1]])

#' We can then write the MOG back out to an .mdl file.
mdlfile.VPC <- paste0(uc,"_VPC.mdl")
writeMogObj(myVPCMOG,mdlfile.VPC)


#' Similarly as above, ddmore "VPC.PsN" function can be used to run a VPC using PsN as target tool
#+ VPC, cache=TRUE
vpcFiles <- VPC.PsN(mdlfile.VPC,samples=20, seed=12345,
		vpcOptions ="-n_simulation=10",
		subfolder="VPC", plot=TRUE) 

#' To replay the visualisation using information from the VPC SO file
pdf(paste0(uc,"_VPC.pdf"))
print(xpose.VPC(vpc.info= file.path("./VPC",vpcFiles@RawResults@DataFiles$PsN_VPC_results$path),
				vpctab= file.path("./VPC",vpcFiles@RawResults@DataFiles$PsN_VPC_vpctab$path),
				main="VPC warfarin"))
dev.off()


#' Simulation using simulx
#' --------------------------------
#' The mlxR package has been developed to visualize and explore models that are encoded in
#' MLXTRAN or PharmML.
#' The ddmore function as.PharmML translates an MDL file (extension .mdl) to its PharmML
#' representation. The output file (extension .xml) is saved in the working directory.
#+ Simulation via simulx
myPharmML <- as.PharmML(mdlfile)

#' Use parameter values from the FOCEI estimation
parValues <- getPopulationParameters(NM.FOCEI,what="estimates")$MLE

#' Simulate for the typical weight of 70. 
#' Recall that logtWT = log(WT/70). 
p <- c(parValues,logtWT=0)

#' Parameter values used in simulation
print(p) 

#' Simulate for a dose of 100 mg given at time 0 into the CENTRAL (iv administration)
#' and a dose of 150 mg given to the CENTRAL (iv dose). Note that we are using COMPARMTMENTS 
#' for dosing processes, which translate to PK macros, and therefore the type option needs to be used
adm1 <- list(type=2, time = 0, amount=100, rate=100) #iv dose
adm2 <- list(type=1, time = 168, amount=150) #oral dose

#' Simulate PK parameters for individuals
ind <- list(name = c('CL','V'))

#' Simulate predicted (CC) and observed concentration values (Y)
f   <- list( name = c('CC'), time = seq(0,to=300,by=1))
y   <- list( name = c('Y'), time = c(0, 0.5, 1, 4, 8, 12, 24, 36, 48,120,
				168,168.5,170,171,174,180,192,216,240,288))

#' Simulate 12 subjects
g <- list( size = 12, level = 'individual')

#' Call simulx
res  <- simulx(model = myPharmML,
		parameter = p,
		group = g,
		treatment = list(adm1,adm2),
		output = list(ind,f,y))

#' Simulated parameter values for each individual
print(res$parameter)

#' Plot simulated results
plot(ggplot() + 
				geom_line(data=res$CC, aes(x=time, y=CC, colour=id)) +
				geom_point(data=res$Y, aes(x=time, y=Y, colour=id)) +
				xlab("time (h)") + ylab("concentration") )

#' Simulate 1000 subjects - with simulx this is a QUICK process!
#+ VPC via simulx
g <- list( size = 1000, level = 'individual')

#' Call simulx
res.1000  <- simulx(model = myPharmML,
		parameter = p,
		group = g,
		treatment = list(adm1,adm2),
		output = list(ind,f,y))

#' Plot prediction intervals with `prctilemlx`. `band` defines the percentile bands displayed:
#' * level = range of values to examine (in %; 100 = full range of values)
#' * number = number of bins within the level range.
print(prctilemlx(res.1000$Y,band=list(number=9, level=90)))

#' Table of the same information
print(prctilemlx(res.1000$Y,band=list(number=10, level=100), plot=FALSE)$y)
